"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const aws_cdk_1 = require("aws-cdk");
const aws_sdk_1 = require("aws-sdk");
const commands_fs_1 = require("@seagull/commands-fs");
const lib = require("../lib");
const provide_asset_folder_1 = require("../provide_asset_folder");
const seagull_app_1 = require("../seagull_app");
const set_aws_credentials_1 = require("../set_aws_credentials");
class SeagullProject {
    constructor(props) {
        this.accountId = props.accountId;
        this.appPath = props.appPath;
        this.branch = props.branch;
        this.mode = props.mode;
        this.profile = props.profile;
        this.region = props.region;
        this.pkgJson = require(`${this.appPath}/package.json`);
    }
    createSeagullApp() {
        return __awaiter(this, void 0, void 0, function* () {
            const suffix = this.mode === 'test' ? `-${this.branch}-${this.mode}` : '';
            const name = `${this.pkgJson.name}${suffix}`;
            const sdk = new aws_cdk_1.SDK({});
            const account = yield sdk.defaultAccount();
            const itemBucketName = yield this.getBucketName();
            const actions = [
                'sts:AssumeRole',
                'logs:CreateLogStream',
                'logs:PutLogEvents',
                'lambda:InvokeFunction',
                'lambda:InvokeAsync',
                'ses:*',
                's3:*',
                'events:*',
            ];
            const aliasConfig = yield checkForAliasConfig(this.pkgJson);
            const appProps = {
                addAssets: true,
                appPath: this.appPath,
                itemsBucket: itemBucketName,
                projectName: name,
                stackProps: { env: { account, region: this.region } },
            };
            yield addResources(this.appPath, itemBucketName);
            const app = new seagull_app_1.SeagullApp(appProps);
            const role = app.stack.addIAMRole('role', 'lambda.amazonaws.com', actions);
            const lambda = app.stack.addUniversalLambda('lambda', this.appPath, role);
            const apiGateway = app.stack.addUniversalApiGateway('api-gateway', lambda);
            app.stack.addCloudfront('cloudfront', { apiGateway, aliasConfig });
            const s3DeploymentNeeded = this.mode === 'prod' || this.branch === 'master';
            const importS3 = () => app.stack.importS3(itemBucketName, role);
            const addS3 = () => app.stack.addS3(itemBucketName, role);
            s3DeploymentNeeded ? addS3() : importS3();
            app.stack.addLogGroup(`/aws/lambda/${name}-lambda-handler`);
            app.stack.addLogGroup(`/${name}/data-log`);
            const cronJson = require(`${this.appPath}/dist/cron.json`);
            cronJson.forEach((rule) => {
                app.stack.addEventRule(rule, lambda);
            });
            return app;
        });
    }
    deployProject() {
        return __awaiter(this, void 0, void 0, function* () {
            this.validate();
            const app = yield this.createSeagullApp();
            yield app.deployStack();
        });
    }
    diffProject() {
        return __awaiter(this, void 0, void 0, function* () {
            const app = yield this.createSeagullApp();
            yield app.diffStack();
        });
    }
    validate() {
        const hasValidProfile = set_aws_credentials_1.setCredsByProfile(this.profile);
        !hasValidProfile && lib.noCredentialsSet();
    }
    getBucketName() {
        return __awaiter(this, void 0, void 0, function* () {
            const accountId = yield getAccountId(this.accountId);
            const prefix = `${this.region}-${accountId}-`;
            const bucketName = `${require(`${this.appPath}/package.json`).name}-items`;
            const suffix = `${this.mode !== 'prod' ? `-${this.mode}` : ''}`;
            return `${prefix}${bucketName}${suffix}`;
        });
    }
}
exports.SeagullProject = SeagullProject;
function checkForAliasConfig(pkgJson) {
    return __awaiter(this, void 0, void 0, function* () {
        const domains = pkgJson.seagull && pkgJson.seagull.domains;
        const needAliases = domains && domains.length > 0;
        return needAliases ? yield getAliasConfig(domains) : undefined;
    });
}
function getAliasConfig(domains) {
    return __awaiter(this, void 0, void 0, function* () {
        const acm = new aws_sdk_1.ACM({ region: 'us-east-1' });
        const params = { CertificateStatuses: ['ISSUED'] };
        const response = yield acm.listCertificates(params).promise();
        const certList = (response && response.CertificateSummaryList) || [];
        const certArns = certList.map(cert => cert.CertificateArn);
        const arns = certArns.filter(arn => arn !== undefined);
        const arnsWithSchemata = yield Promise.all(arns.map(getCertificateDomains));
        return lib.findAliasConfig(arnsWithSchemata, domains);
    });
}
function getCertificateDomains(acmCertRef) {
    return __awaiter(this, void 0, void 0, function* () {
        const acm = new aws_sdk_1.ACM({ region: 'us-east-1' });
        const params = { CertificateArn: acmCertRef };
        const cert = (yield acm.describeCertificate(params).promise()).Certificate;
        const names = (cert && cert.SubjectAlternativeNames) || [];
        return { acmCertRef, names };
    });
}
function getAccountId(accountId) {
    return __awaiter(this, void 0, void 0, function* () {
        return accountId || (yield new aws_sdk_1.STS().getCallerIdentity().promise()).Account;
    });
}
function addResources(appPath, itemsBucket) {
    return __awaiter(this, void 0, void 0, function* () {
        const provideAssetFolder = new provide_asset_folder_1.ProvideAssetFolder(appPath);
        yield provideAssetFolder.execute();
        itemsBucket && (yield replaceS3BucketName(appPath, itemsBucket));
    });
}
function replaceS3BucketName(appPath, itemsBucket) {
    return __awaiter(this, void 0, void 0, function* () {
        const lambdaPath = `.seagull/deploy/dist/assets/backend/lambda.js`;
        const lambdaFile = `${appPath}/${lambdaPath}`;
        const lambda = yield new commands_fs_1.FS.ReadFile(lambdaFile).execute();
        const lambdaWithBucketName = lambda.replace('demo-bucket', itemsBucket);
        yield new commands_fs_1.FS.WriteFile(lambdaFile, lambdaWithBucketName).execute();
    });
}
//# sourceMappingURL=seagull_project.js.map